@@ -21,7 +21,7 @@ package lru
 
 import (
 	"fmt"
-	"github.com/IrineSistiana/mosdns/v4/pkg/list"
+	"github.com/IrineSistiana/mosdns/v5/pkg/list"
 )
 
 type LRU[K comparable, V any] struct {
@@ -116,6 +116,11 @@ func (q *LRU[K, V]) Clean(f func(key K, v V) (remove bool)) (removed int) {
 	return removed
 }
 
+func (q *LRU[K, V]) Flush() {
+	q.l = list.New[KV[K, V]]()
+	q.m = make(map[K]*list.Elem[KV[K, V]])
+}
+
 func (q *LRU[K, V]) Get(key K) (v V, ok bool) {
 	e, ok := q.m[key]
 	if !ok {
