@@ -22,53 +22,63 @@ package doh
 import (
 	"context"
 	"encoding/base64"
+	"encoding/binary"
 	"fmt"
+	"io"
+	"net/http"
+	urlpkg "net/url"
+	"time"
+
+	"github.com/IrineSistiana/mosdns/v5/pkg/dnsutils"
 	"github.com/IrineSistiana/mosdns/v5/pkg/pool"
 	"github.com/IrineSistiana/mosdns/v5/pkg/utils"
 	"github.com/miekg/dns"
-	"io"
-	"net/http"
-	"strings"
-	"time"
+	"go.uber.org/zap"
 )
 
 const (
-	defaultDoHTimeout = time.Second * 5
+	defaultDoHTimeout = time.Second * 6
 )
 
+var nopLogger = zap.NewNop()
+
 // Upstream is a DNS-over-HTTPS (RFC 8484) upstream.
 type Upstream struct {
-	// EndPoint is the DoH server URL.
-	EndPoint string
-	// Client is a http.Client that sends http requests.
-	Client *http.Client
-
-	// AddOnCloser will be closed when Upstream is closed.
-	AddOnCloser io.Closer
+	rt          http.RoundTripper
+	logger      *zap.Logger // non-nil
+	urlTemplate *urlpkg.URL
+	reqTemplate *http.Request
 }
 
-func (u *Upstream) CloseIdleConnections() {
-	u.Client.CloseIdleConnections()
-}
-
-func (u *Upstream) Close() error {
-	u.Client.CloseIdleConnections()
-	if u.AddOnCloser != nil {
-		u.AddOnCloser.Close()
+func NewUpstream(endPoint string, rt http.RoundTripper, logger *zap.Logger) (*Upstream, error) {
+	req, err := http.NewRequest(http.MethodGet, endPoint, nil)
+	if err != nil {
+		return nil, fmt.Errorf("failed to parse http request, %w", err)
 	}
-	return nil
+
+	req.Header["Accept"] = []string{"application/dns-message"}
+	req.Header["User-Agent"] = nil // Don't let go http send a default user agent header.
+
+	if logger == nil {
+		logger = nopLogger
+	}
+	return &Upstream{
+		rt:          rt,
+		logger:      logger,
+		urlTemplate: req.URL,
+		reqTemplate: req,
+	}, nil
 }
 
 var (
 	bufPool4k = pool.NewBytesBufPool(4096)
 )
 
-func (u *Upstream) ExchangeContext(ctx context.Context, q *dns.Msg) (*dns.Msg, error) {
-	wire, buf, err := pool.PackBuffer(q)
-	if err != nil {
-		return nil, fmt.Errorf("failed to pack query msg, %w", err)
-	}
-	defer pool.ReleaseBuf(buf)
+func (u *Upstream) ExchangeContext(ctx context.Context, q []byte) (*[]byte, error) {
+	bp := pool.GetBuf(len(q))
+	defer pool.ReleaseBuf(bp)
+	wire := *bp
+	copy(wire, q)
 
 	// In order to maximize HTTP cache friendliness, DoH clients using media
 	// formats that include the ID field from the DNS message header, such
@@ -78,61 +88,55 @@ func (u *Upstream) ExchangeContext(ctx context.Context, q *dns.Msg) (*dns.Msg, e
 	wire[0] = 0
 	wire[1] = 0
 
-	urlLen := len(u.EndPoint) + 5 + base64.RawURLEncoding.EncodedLen(len(wire))
-	urlBuf := make([]byte, urlLen)
+	queryLen := 4 + base64.RawURLEncoding.EncodedLen(len(wire))
+	queryBuf := make([]byte, queryLen)
 
 	p := 0
-	p += copy(urlBuf[p:], u.EndPoint)
-	// A simple way to check whether the endpoint already has a parameter.
-	if strings.LastIndexByte(u.EndPoint, '?') >= 0 {
-		p += copy(urlBuf[p:], "&dns=")
-	} else {
-		p += copy(urlBuf[p:], "?dns=")
-	}
+	p += copy(queryBuf, "dns=")
 
 	// Padding characters for base64url MUST NOT be included.
 	// See: https://tools.ietf.org/html/rfc8484#section-6.
-	base64.RawURLEncoding.Encode(urlBuf[p:], wire)
+	base64.RawURLEncoding.Encode(queryBuf[p:], wire)
 
-	type result struct {
-		r   *dns.Msg
+	type res struct {
+		r   *[]byte
 		err error
 	}
 
-	resChan := make(chan *result, 1)
+	resChan := make(chan res, 1)
 	go func() {
-		// We overwrite the ctx with a fixed timout context here.
+		// We overwrite the ctx with a fixed timeout context here.
 		// Because the http package may close the underlay connection
 		// if the context is done before the query is completed. This
 		// reduces the connection reuse efficiency.
 		ctx, cancel := context.WithTimeout(context.Background(), defaultDoHTimeout)
 		defer cancel()
-		r, err := u.exchange(ctx, utils.BytesToStringUnsafe(urlBuf))
-		resChan <- &result{r: r, err: err}
+		r, err := u.exchange(ctx, utils.BytesToStringUnsafe(queryBuf))
+		if err != nil {
+			u.logger.Check(zap.WarnLevel, "exchange failed").Write(zap.Error(err))
+		}
+		resChan <- res{r: r, err: err}
 	}()
 
 	select {
 	case <-ctx.Done():
-		return nil, ctx.Err()
+		return nil, context.Cause(ctx)
 	case res := <-resChan:
 		r := res.r
 		err := res.err
 		if r != nil {
-			r.Id = q.Id
+			binary.BigEndian.PutUint16(*r, binary.BigEndian.Uint16(q))
 		}
 		return r, err
 	}
 }
 
-func (u *Upstream) exchange(ctx context.Context, url string) (*dns.Msg, error) {
-	req, err := http.NewRequestWithContext(ctx, http.MethodGet, url, nil)
-	if err != nil {
-		return nil, fmt.Errorf("interal err: NewRequestWithContext: %w", err)
-	}
-
-	req.Header["Accept"] = []string{"application/dns-message"}
-	req.Header["User-Agent"] = nil // Don't let go http send a default user agent header.
-	resp, err := u.Client.Do(req)
+func (u *Upstream) exchange(ctx context.Context, dnsQuery string) (*[]byte, error) {
+	req := u.reqTemplate.WithContext(ctx)
+	req.URL = new(urlpkg.URL)
+	*req.URL = *u.urlTemplate
+	req.URL.RawQuery = dnsQuery
+	resp, err := u.rt.RoundTrip(req)
 	if err != nil {
 		return nil, fmt.Errorf("http request failed: %w", err)
 	}
@@ -153,10 +157,10 @@ func (u *Upstream) exchange(ctx context.Context, url string) (*dns.Msg, error) {
 	if err != nil {
 		return nil, fmt.Errorf("failed to read http body: %w", err)
 	}
-
-	r := new(dns.Msg)
-	if err := r.Unpack(bb.Bytes()); err != nil {
-		return nil, fmt.Errorf("failed to unpack http body: %w", err)
+	if bb.Len() < dnsutils.DnsHeaderLen {
+		return nil, dnsutils.ErrPayloadTooSmall
 	}
-	return r, nil
+	payload := pool.GetBuf(bb.Len())
+	copy(*payload, bb.Bytes())
+	return payload, nil
 }
