@@ -21,12 +21,11 @@ package plugin
 
 // data providers
 import (
+	// data provider
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/data_provider/domain_set"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/data_provider/ip_set"
-)
 
-// matches
-import (
+	// matcher
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/client_ip"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/cname"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/env"
@@ -39,39 +38,37 @@ import (
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/random"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/rcode"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/resp_ip"
-)
+	_ "github.com/IrineSistiana/mosdns/v5/plugin/matcher/string_exp"
 
-// executables
-import (
+	// executable
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/arbitrary"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/black_hole"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/cache"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/debug_print"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/drop_resp"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/dual_selector"
-	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/ecs"
+	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/ecs_handler"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/forward"
+	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/forward_edns0opt"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/hosts"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/ipset"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/metrics_collector"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/nftset"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/query_summary"
+	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/rate_limiter"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/redirect"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/reverse_lookup"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/sequence"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/sequence/fallback"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/sleep"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/executable/ttl"
-)
 
-// other
-import (
-	_ "github.com/IrineSistiana/mosdns/v5/plugin/mark" // executable and matcher
-)
+	// executable and matcher
+	_ "github.com/IrineSistiana/mosdns/v5/plugin/mark"
 
-// servers
-import (
+	// server
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/server/http_server"
+	_ "github.com/IrineSistiana/mosdns/v5/plugin/server/quic_server"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/server/tcp_server"
 	_ "github.com/IrineSistiana/mosdns/v5/plugin/server/udp_server"
 )
