@@ -24,7 +24,11 @@ import (
 	"crypto/tls"
 	"errors"
 	"fmt"
+	"strings"
+	"time"
+
 	"github.com/IrineSistiana/mosdns/v5/coremain"
+	"github.com/IrineSistiana/mosdns/v5/pkg/pool"
 	"github.com/IrineSistiana/mosdns/v5/pkg/query_context"
 	"github.com/IrineSistiana/mosdns/v5/pkg/upstream"
 	"github.com/IrineSistiana/mosdns/v5/pkg/utils"
@@ -32,8 +36,6 @@ import (
 	"github.com/miekg/dns"
 	"github.com/prometheus/client_golang/prometheus"
 	"go.uber.org/zap"
-	"strings"
-	"time"
 )
 
 const PluginType = "forward"
@@ -57,22 +59,27 @@ type Args struct {
 	SoMark       int    `yaml:"so_mark"`
 	BindToDevice string `yaml:"bind_to_device"`
 	Bootstrap    string `yaml:"bootstrap"`
+	BootstrapVer int    `yaml:"bootstrap_version"`
 }
 
 type UpstreamConfig struct {
-	Tag                string `yaml:"tag"`
-	Addr               string `yaml:"addr"` // Required.
-	DialAddr           string `yaml:"dial_addr"`
-	IdleTimeout        int    `yaml:"idle_timeout"`
-	MaxConns           int    `yaml:"max_conns"`
-	EnablePipeline     bool   `yaml:"enable_pipeline"`
-	EnableHTTP3        bool   `yaml:"enable_http3"`
-	InsecureSkipVerify bool   `yaml:"insecure_skip_verify"`
+	Tag         string `yaml:"tag"`
+	Addr        string `yaml:"addr"` // Required.
+	DialAddr    string `yaml:"dial_addr"`
+	IdleTimeout int    `yaml:"idle_timeout"`
+
+	// Deprecated: This option has no affect.
+	// TODO: (v6) Remove this option.
+	MaxConns           int  `yaml:"max_conns"`
+	EnablePipeline     bool `yaml:"enable_pipeline"`
+	EnableHTTP3        bool `yaml:"enable_http3"`
+	InsecureSkipVerify bool `yaml:"insecure_skip_verify"`
 
 	Socks5       string `yaml:"socks5"`
 	SoMark       int    `yaml:"so_mark"`
 	BindToDevice string `yaml:"bind_to_device"`
 	Bootstrap    string `yaml:"bootstrap"`
+	BootstrapVer int    `yaml:"bootstrap_version"`
 }
 
 func Init(bp *coremain.BP, args any) (any, error) {
@@ -94,7 +101,7 @@ type Forward struct {
 	args *Args
 
 	logger       *zap.Logger
-	us           map[*upstreamWrapper]struct{}
+	us           []*upstreamWrapper
 	tag2Upstream map[string]*upstreamWrapper // for fast tag lookup only.
 }
 
@@ -116,7 +123,6 @@ func NewForward(args *Args, opt Opts) (*Forward, error) {
 	f := &Forward{
 		args:         args,
 		logger:       opt.Logger,
-		us:           make(map[*upstreamWrapper]struct{}),
 		tag2Upstream: make(map[string]*upstreamWrapper),
 	}
 
@@ -125,6 +131,7 @@ func NewForward(args *Args, opt Opts) (*Forward, error) {
 		utils.SetDefaultUnsignNum(&c.SoMark, args.SoMark)
 		utils.SetDefaultString(&c.BindToDevice, args.BindToDevice)
 		utils.SetDefaultString(&c.Bootstrap, args.Bootstrap)
+		utils.SetDefaultUnsignNum(&c.BootstrapVer, args.BootstrapVer)
 	}
 
 	for i, c := range args.Upstreams {
@@ -133,17 +140,17 @@ func NewForward(args *Args, opt Opts) (*Forward, error) {
 		}
 		applyGlobal(&c)
 
-		uw := newWrapper(c, opt.MetricsTag)
+		uw := newWrapper(i, c, opt.MetricsTag)
 		uOpt := upstream.Opt{
 			DialAddr:       c.DialAddr,
 			Socks5:         c.Socks5,
 			SoMark:         c.SoMark,
 			BindToDevice:   c.BindToDevice,
 			IdleTimeout:    time.Duration(c.IdleTimeout) * time.Second,
-			MaxConns:       c.MaxConns,
 			EnablePipeline: c.EnablePipeline,
 			EnableHTTP3:    c.EnableHTTP3,
 			Bootstrap:      c.Bootstrap,
+			BootstrapVer:   c.BootstrapVer,
 			TLSConfig: &tls.Config{
 				InsecureSkipVerify: c.InsecureSkipVerify,
 				ClientSessionCache: tls.NewLRUClientSessionCache(4),
@@ -158,7 +165,7 @@ func NewForward(args *Args, opt Opts) (*Forward, error) {
 			return nil, fmt.Errorf("failed to init upstream #%d: %w", i, err)
 		}
 		uw.u = u
-		f.us[uw] = struct{}{}
+		f.us = append(f.us, uw)
 
 		if len(c.Tag) > 0 {
 			if _, dup := f.tag2Upstream[c.Tag]; dup {
@@ -173,7 +180,7 @@ func NewForward(args *Args, opt Opts) (*Forward, error) {
 }
 
 func (f *Forward) RegisterMetricsTo(r prometheus.Registerer) error {
-	for wu := range f.us {
+	for _, wu := range f.us {
 		// Only register metrics for upstream that has a tag.
 		if len(wu.cfg.Tag) == 0 {
 			continue
@@ -196,17 +203,16 @@ func (f *Forward) Exec(ctx context.Context, qCtx *query_context.Context) (err er
 
 // QuickConfigureExec format: [upstream_tag]...
 func (f *Forward) QuickConfigureExec(args string) (any, error) {
-	var us map[*upstreamWrapper]struct{}
+	var us []*upstreamWrapper
 	if len(args) == 0 { // No args, use all upstreams.
 		us = f.us
 	} else { // Pick up upstreams by tags.
-		us = make(map[*upstreamWrapper]struct{})
 		for _, tag := range strings.Fields(args) {
 			u := f.tag2Upstream[tag]
 			if u == nil {
 				return nil, fmt.Errorf("cannot find upstream by tag %s", tag)
 			}
-			us[u] = struct{}{}
+			us = append(us, u)
 		}
 	}
 	var execFunc sequence.ExecutableFunc = func(ctx context.Context, qCtx *query_context.Context) error {
@@ -221,28 +227,33 @@ func (f *Forward) QuickConfigureExec(args string) (any, error) {
 }
 
 func (f *Forward) Close() error {
-	for u := range f.us {
-		_ = (*u).Close()
+	for _, u := range f.us {
+		_ = u.Close()
 	}
 	return nil
 }
 
-func (f *Forward) exchange(ctx context.Context, qCtx *query_context.Context, us map[*upstreamWrapper]struct{}) (*dns.Msg, error) {
+func (f *Forward) exchange(ctx context.Context, qCtx *query_context.Context, us []*upstreamWrapper) (*dns.Msg, error) {
 	if len(us) == 0 {
 		return nil, errors.New("no upstream to exchange")
 	}
 
-	mcq := f.args.Concurrent
-	if mcq <= 0 {
-		mcq = 1
+	queryPayload, err := pool.PackBuffer(qCtx.Q())
+	if err != nil {
+		return nil, err
 	}
-	if mcq > maxConcurrentQueries {
-		mcq = maxConcurrentQueries
+	defer pool.ReleaseBuf(queryPayload)
+
+	concurrent := f.args.Concurrent
+	if concurrent <= 0 {
+		concurrent = 1
+	}
+	if concurrent > maxConcurrentQueries {
+		concurrent = maxConcurrentQueries
 	}
 
 	type res struct {
 		r   *dns.Msg
-		u   *upstreamWrapper
 		err error
 	}
 
@@ -250,56 +261,60 @@ func (f *Forward) exchange(ctx context.Context, qCtx *query_context.Context, us
 	done := make(chan struct{})
 	defer close(done)
 
-	qc := qCtx.Q().Copy()
-	uqid := qCtx.Id()
-	sent := 0
-	for u := range us {
-		if sent > mcq {
-			break
-		}
-		sent++
-
-		u := u
-		go func() {
-			// Give each upstream a fixed timeout to finsh the query.
+	for i := 0; i < concurrent; i++ {
+		u := randPick(us)
+		qc := copyPayload(queryPayload)
+		go func(uqid uint32, question dns.Question) {
+			defer pool.ReleaseBuf(qc)
+			// Give each upstream a fixed timeout to finish the query.
 			upstreamCtx, cancel := context.WithTimeout(context.Background(), queryTimeout)
 			defer cancel()
-			r, err := u.ExchangeContext(upstreamCtx, qc)
+
+			var r *dns.Msg
+			respPayload, err := u.ExchangeContext(upstreamCtx, *qc)
 			if err != nil {
 				f.logger.Warn(
 					"upstream error",
 					zap.Uint32("uqid", uqid),
-					zap.Inline((*queryInfo)(qc)),
+					zap.String("qname", question.Name),
+					zap.Uint16("qclass", question.Qclass),
+					zap.Uint16("qtype", question.Qtype),
 					zap.String("upstream", u.name()),
 					zap.Error(err),
 				)
+			} else {
+				r = new(dns.Msg)
+				err = r.Unpack(*respPayload)
+				pool.ReleaseBuf(respPayload)
+				if err != nil {
+					r = nil
+				}
 			}
 			select {
-			case resChan <- res{r: r, err: err, u: u}:
+			case resChan <- res{r: r, err: err}:
 			case <-done:
 			}
-		}()
+		}(qCtx.Id(), qCtx.QQuestion())
 	}
 
-	es := new(utils.Errors)
-	for i := 0; i < sent; i++ {
+	for i := 0; i < concurrent; i++ {
 		select {
 		case res := <-resChan:
-			r, u, err := res.r, res.u, res.err
+			r, err := res.r, res.err
 			if err != nil {
-				es.Append(&upstreamErr{
-					upstreamName: u.name(),
-					err:          err,
-				})
+				continue
+			}
+
+			// Retry until the last
+			if i < concurrent-1 && r.Rcode != dns.RcodeSuccess && r.Rcode != dns.RcodeNameError {
 				continue
 			}
 			return r, nil
 		case <-ctx.Done():
-			es.Append(fmt.Errorf("exchange: %w", ctx.Err()))
-			return nil, es
+			return nil, context.Cause(ctx)
 		}
 	}
-	return nil, es
+	return nil, errors.New("all upstream servers failed")
 }
 
 func quickSetup(bq sequence.BQ, s string) (any, error) {
