@@ -20,14 +20,18 @@
 package tcp_server
 
 import (
+	"context"
 	"crypto/tls"
 	"fmt"
+	"net"
+	"strings"
+	"time"
+
 	"github.com/IrineSistiana/mosdns/v5/coremain"
 	"github.com/IrineSistiana/mosdns/v5/pkg/server"
 	"github.com/IrineSistiana/mosdns/v5/pkg/utils"
 	"github.com/IrineSistiana/mosdns/v5/plugin/server/server_utils"
-	"net"
-	"time"
+	"go.uber.org/zap"
 )
 
 const PluginType = "tcp_server"
@@ -69,9 +73,6 @@ func StartServer(bp *coremain.BP, args *Args) (*TcpServer, error) {
 		return nil, fmt.Errorf("failed to init dns handler, %w", err)
 	}
 
-	serverOpts := server.TCPServerOpts{Logger: bp.L(), DNSHandler: dh, IdleTimeout: time.Duration(args.IdleTimeout) * time.Second}
-	s := server.NewTCPServer(serverOpts)
-
 	// Init tls
 	var tc *tls.Config
 	if len(args.Key)+len(args.Cert) > 0 {
@@ -81,17 +82,28 @@ func StartServer(bp *coremain.BP, args *Args) (*TcpServer, error) {
 		}
 	}
 
-	l, err := net.Listen("tcp", args.Listen)
+	socketOpt := server_utils.ListenerSocketOpts{
+		SO_REUSEPORT: true,
+		SO_RCVBUF:    64 * 1024,
+	}
+	lc := net.ListenConfig{Control: server_utils.ListenerControl(socketOpt)}
+	listenerNetwork := "tcp"
+	if strings.HasPrefix(args.Listen, "@") {
+		listenerNetwork = "unix"
+	}
+	l, err := lc.Listen(context.Background(), listenerNetwork, args.Listen)
 	if err != nil {
 		return nil, fmt.Errorf("failed to listen socket, %w", err)
 	}
 	if tc != nil {
 		l = tls.NewListener(l, tc)
 	}
+	bp.L().Info("tcp server started", zap.Stringer("addr", l.Addr()), zap.Bool("tls", tc != nil))
 
 	go func() {
 		defer l.Close()
-		err := s.ServeTCP(l)
+		serverOpts := server.TCPServerOpts{Logger: bp.L(), IdleTimeout: time.Duration(args.IdleTimeout) * time.Second}
+		err := server.ServeTCP(l, dh, serverOpts)
 		bp.M().GetSafeClose().SendCloseSignal(err)
 	}()
 	return &TcpServer{
